-- View: public.access_review_details_v

-- DROP VIEW public.access_review_details_v;

CREATE OR REPLACE VIEW public.access_review_details_v AS 
 SELECT DISTINCT item.id AS review_item_id,
    item.last_override_action_id AS last_override_action,
    instance.id AS review_instance_id,
    instance.status AS review_status,
    instance.start_date,
    instance.end_date,
    instance.valid_to_date,
    instance.item_count,
    multi_owner.review_owners,
    multi_reviewer.reviewer_ids AS reviewer_id,
        CASE
            WHEN item.final_cert_by_unique_user_id IS NOT NULL THEN final_reviewer.display_name::text
            ELSE multi_reviewer.reviewers
        END AS reviewer_display_name,
    reassigned_by_history.reviewers AS reassigned_by_user_disp_name,
    reassigned_from_history.reviewers AS reassigned_from_user_disp_name,
    reassigned_by_history.reassigned_date,
    defn.name AS review_name,
    substr(defn.description, 0, 4000) AS description,
    defn.review_type,
    defn.next_schedule_date,
        CASE
            WHEN item.item_type::text = 'ACCOUNT'::text AND usr.id IS NULL THEN shacc.suser
            ELSE usr.id
        END AS user_id,
        CASE
            WHEN item.item_type::text = 'ACCOUNT'::text AND usr.display_name IS NULL THEN shacc.display_name::text
            ELSE usr.display_name::text
        END AS user_display_name,
    action.action AS reviewer_action,
    action.review_order,
    action.reviewer_comment,
    perm.id AS permission_id,
    perm.name AS permission_name,
    acct.id AS account_id,
    acct.account_id AS account_native_id,
    acct.name AS account_name,
    acct.relation_to_user_type AS account_relation,
    app.id AS application_id,
    app_ds.name::text AS application_name,
    item.final_decision,
    item.final_decision_time,
    role.id AS role_id,
    role.name AS role_name,
    item.item_type,
    brole.name AS brole_name,
    action.fulfillment_instructions,
    esc_reviewer.display_name
   FROM review_item item
     JOIN review_instance instance ON item.review_instance_id = instance.id
     JOIN review_definition defn ON instance.review_definition = defn.id
     LEFT JOIN suser usr ON usr.unique_user_id::text = item.unique_user_id::text AND usr.effective = true
     LEFT JOIN spermission perm ON perm.unique_permission_id::text = item.unique_permission_id::text AND perm.effective = true
     LEFT JOIN saccount acct ON acct.unique_account_id::text = item.unique_account_id::text AND acct.effective = true
     LEFT JOIN application app ON app.id =
        CASE
            WHEN perm.id IS NULL THEN acct.application
            ELSE perm.application
        END
     LEFT JOIN business_role brole ON brole.id = item.business_role_id
     LEFT JOIN role_policy role ON role.id = item.role_id
     LEFT JOIN data_source app_ds ON app_ds.id = app.id
     LEFT JOIN review_item_action action ON action.review_item_id = item.id AND action.action::text <> 'REASSIGNED'::text
     LEFT JOIN reviewer_instance inst2 ON instance.id = inst2.review_instance_id AND inst2.type::text = 'RO'::text
     LEFT JOIN reviewer_instance inst3 ON instance.id = inst3.review_instance_id
     LEFT JOIN ( SELECT DISTINCT item_1.id AS item_id,
            string_agg(multi_reviewers.id::text, ' | '::text ORDER BY multi_reviewers.id) AS reviewer_ids,
            string_agg(multi_reviewers.unique_user_id::text, ' | '::text ORDER BY multi_reviewers.unique_user_id) AS reviewer_unique_ids,
            string_agg(multi_reviewers.display_name::text, ' | '::text ORDER BY multi_reviewers.display_name) AS reviewers
           FROM review_item item_1
             LEFT JOIN reviewer_user_instance rrui ON rrui.reviewer_instance_id = item_1.curr_reviewer_instance_id
             LEFT JOIN suser multi_reviewers ON rrui.unique_user_id::text = multi_reviewers.unique_user_id::text AND multi_reviewers.effective = true
          GROUP BY item_1.id) multi_reviewer ON multi_reviewer.item_id = item.id
     LEFT JOIN suser final_reviewer ON item.final_cert_by_unique_user_id::text = final_reviewer.unique_user_id::text AND final_reviewer.effective = true
     LEFT JOIN suser acting_reviewer ON action.acted_by_user_unique_id::text = acting_reviewer.unique_user_id::text AND acting_reviewer.effective = true
     LEFT JOIN review_item_action reassigned ON reassigned.review_item_id = item.id AND reassigned.action::text = 'REASSIGNED'::text AND reassigned.reassigned_to_user_unique_id::text ~~ (('%'::text || multi_reviewer.reviewer_unique_ids) || '%'::text)
     LEFT JOIN suser curr_reviewer ON curr_reviewer.unique_user_id::text ~~ (('%'::text || multi_reviewer.reviewer_unique_ids) || '%'::text) AND curr_reviewer.effective = true
     LEFT JOIN ( SELECT DISTINCT ria.review_item_id,
            string_agg(usr_1.display_name::text, ' | '::text ORDER BY ria.assigned) AS reviewers
           FROM review_item_action ria
             LEFT JOIN reviewer_instance ri ON ri.id = ria.reviewer_instance_id
             LEFT JOIN suser usr_1 ON ri.single_reviewer_unique_id::text = usr_1.unique_user_id::text AND usr_1.effective = true
          WHERE ria.action::text = 'REASSIGNED'::text
          GROUP BY ria.review_item_id) reassigned_from_history ON reassigned_from_history.review_item_id = item.id
     LEFT JOIN ( SELECT DISTINCT ria.review_item_id,
            string_agg(usr_1.display_name::text, ' | '::text ORDER BY ria.assigned) AS reviewers,
            string_agg(to_char(ria.assigned, 'YYYY-MM-DD HH24:MI:SS'::text), ' | '::text ORDER BY ria.assigned) AS reassigned_date
           FROM review_item_action ria
             LEFT JOIN suser usr_1 ON ria.reassigned_by_user_unique_id::text = usr_1.unique_user_id::text AND usr_1.effective = true
          WHERE ria.action::text = 'REASSIGNED'::text
          GROUP BY ria.review_item_id) reassigned_by_history ON reassigned_by_history.review_item_id = item.id
     LEFT JOIN shared_accounts_v shacc ON shacc.unique_account_id::text = acct.unique_account_id::text
     LEFT JOIN ( SELECT DISTINCT defn_1.name AS review_name,
            ri.id AS rid,
            string_agg(s.display_name::text, ' | '::text ORDER BY s.display_name) AS review_owners
           FROM review_instance ri
             JOIN review_definition defn_1 ON ri.review_definition = defn_1.id
             LEFT JOIN reviewer_instance rev_owner ON ri.id = rev_owner.review_instance_id AND rev_owner.type::text = 'RO'::text
             LEFT JOIN reviewer_user_instance rui ON rui.reviewer_instance_id = rev_owner.id
             LEFT JOIN suser s ON rui.unique_user_id::text = s.unique_user_id::text AND s.effective = true
          GROUP BY defn_1.name, ri.id) multi_owner ON multi_owner.rid = item.review_instance_id
     LEFT JOIN ( SELECT esc_reviewers.review_instance_id,
            string_agg(esc_reviewers.display_name::text, ' | '::text) AS display_name
           FROM ( SELECT DISTINCT u.display_name,
                    rri.review_instance_id
                   FROM review_item ri
                     JOIN reviewer_instance rri ON ri.curr_reviewer_instance_id = rri.id AND rri.type::text = 'EO'::text
                     JOIN reviewer_user_instance rriu ON rriu.reviewer_instance_id = rri.id
                     JOIN suser u ON u.unique_user_id::text = rriu.unique_user_id::text AND u.effective = true) esc_reviewers
          GROUP BY esc_reviewers.review_instance_id) esc_reviewer ON esc_reviewer.review_instance_id = item.review_instance_id
  WHERE defn.deleted = false AND (acting_reviewer.unique_user_id::text ~~ (('%'::text || multi_reviewer.reviewer_unique_ids) || '%'::text) OR acting_reviewer.unique_user_id IS NULL AND action.review_order = (( SELECT DISTINCT min(act.review_order) AS min
           FROM review_item_action act
          WHERE (act.action::text = 'PENDING'::text OR act.action::text = 'ESCALATE'::text) AND act.review_item_id = item.id)) OR acting_reviewer.unique_user_id IS NOT NULL AND action.shared_queue = true) AND (action.action::text <> 'PENDING'::text OR item.final_decision IS NULL)
  ORDER BY item.id, action.review_order;

ALTER TABLE public.access_review_details_v
  OWNER TO igops;
GRANT ALL ON TABLE public.access_review_details_v TO igops;
GRANT SELECT ON TABLE public.access_review_details_v TO ig_report_role;
